

require File.join(File.dirname(__FILE__), '../..', "tomte_protocol_spec_helper")

describe "A Dialog, regarding transport reliability" do

  #
  # Environment
  #

  include EventedSpec::AMQPSpec
  default_options AMQP_OPTS
  default_timeout 5


  def close_and_done
    delay(1) {
      @server.close { @client.close {  @ep.disconnect! { done }}}}
  end

  amqp_after do

  end


  describe "on an unreliable transport" do

    amqp_before do
      @ep  = Tomte::Protocol::Endpoint.new(:bus => :test)

      @client = Tomte::Protocol::Dialog::Client.new(@ep,
                               :exchange => "tomte.tests.integration.dialog",
                               :reliable => false,
                               :exchange_options => {:auto_delete => true})

      @server = Tomte::Protocol::Dialog::Server.new(@ep,
                              :queue_options => {:auto_delete => true} )
      @server.append_callback(:entity_ready) do |s|
        s.declare_exchange("tomte.tests.integration.dialog", :direct, :auto_delete => true) {|ex|
          s.bind(ex, :routing_key => "messages.all")
        }
      end

    end

    it 'an undeliverable message is not delivered, delivers an exception' do
      events   = Array.new

      @server.on_call do |metadata, payload|

        expect(payload).to eq "ping"
        events << :received
        "pong"
      end

      @ep.connect! do
        @server.connect! do |s|
          s.consume

          @client.connect! do |c|
            delay(0.5) do
              c.call("ping",
                     :routing_key => "messages.non.existent",
                     ) do |metadata, response|

                expect(response).to be_a_kind_of Tomte::Protocol::TransportFailure
                expect(events).not_to include(:received)
                close_and_done
              end
            end
          end
        end
      end
    end

    it 'a deliverable message raises error if the server times out' do
      events   = Array.new

      @server.on_call do |metadata, payload|

       expect(payload).to eq "ping"
       events << :received

       # ask async processing then do not return
       throw :_async
      end


      @ep.connect! do
        @server.connect! do |s|
          s.consume
          @client.connect! do |c|
           delay(0.5) do
             c.call("ping",
                    :routing_key => "messages.all",
                    :request_timeout => 2,
                    ) do |metadata, response|

               expect(response).to be_a_kind_of Tomte::Protocol::Timeout

               expect(events).to include(:received)
               close_and_done
             end
           end
          end
        end
      end
    end
  end


  describe "on a reliable transport" do

    amqp_before do
      @ep  = Tomte::Protocol::Endpoint.new(:bus => :test)

      @client = Tomte::Protocol::Dialog::Client.new(@ep,
                               :exchange => "tomte.tests.integration.dialog",
                               :reliable => true,
                               :exchange_options => {:auto_delete => true})

      @server = Tomte::Protocol::Dialog::Server.new(@ep,
                              :queue_options => {:auto_delete => true} )
      @server.append_callback(:entity_ready) do |s|
        s.declare_exchange("tomte.tests.integration.dialog", :direct, :auto_delete => true) {|ex|
          s.bind(ex, :routing_key => "messages.all")
        }
      end

    end


    it 'a deliverable message is delivered, acknowledged, and delivered back' do
      events   = Array.new

      @server.on_call do |metadata, payload|

        expect(payload).to eq "ping"
        events << :received
        "pong"
      end

      errback = proc {
        events << :error
      }

      on_ack = proc {
        events << :confirmed
      }

      @ep.connect! do
        @server.connect! do |s|
          s.consume

          @client.connect! do |c|
            delay(0.5) do
              c.call("ping",
                     :routing_key => "messages.all",
                     :errback => errback,
                     :on_ack => on_ack,
                     ) do |metadata, response|


                expect(response).to eq "pong"
                expect(events).to include(:received)
                expect(events).to include(:confirmed)
                expect(events).not_to include(:error)

                close_and_done
              end
            end
          end
        end
      end
    end


    it 'an undeliverable message is not delivered, raises error, and delivers an exception' do
      events   = Array.new

      @server.on_call do |metadata, payload|

        expect(payload).to eq "ping"
        events << :received
        "pong"
      end

      errback = proc {
        events << :error
      }

      on_ack = proc {
        events << :confirmed
      }

      @ep.connect! do
        @server.connect! do |s|
          s.consume

          @client.connect! do |c|
            delay(0.5) do
              c.call("ping",
                     :routing_key => "messages.non.existent",
                     :errback => errback,
                     :on_ack => on_ack,
                     ) do |metadata, response|

                expect(response).to be_a_kind_of Tomte::Protocol::TransportFailure

                # the callback is fired as soon the return message is received.
                # the errback is fired when the *ack* is received, so it might be some time later
                delay(1) {
                  expect(events).not_to include(:received)
                  expect(events).not_to include(:confirmed)
                  expect(events).to include(:error)

                  close_and_done
                }
              end
            end
          end
        end
      end
    end

    it 'a deliverable message is acked, then raises error if the server times out' do
      events   = Array.new

      @server.on_call do |metadata, payload|

        expect(payload).to eq "ping"
        events << :received

        # ask async processing then do not return
        throw :_async
      end

      errback = proc {
        events << :error
      }

      on_ack = proc {
        events << :confirmed
      }

      @ep.connect! do
        @server.connect! do |s|
          s.consume

          @client.connect! do |c|
            delay(0.5) do
              c.call("ping",
                     :routing_key => "messages.all",
                     :errback => errback,
                     :on_ack => on_ack,
                     :request_timeout => 2,
                     ) do |metadata, response|

                expect(response).to be_a_kind_of Tomte::Protocol::Timeout
                expect(events).to include(:received)
                expect(events).to include(:confirmed)
                expect(events).not_to include(:error)

                close_and_done

              end
            end
          end
        end
      end
    end

  end
end