require File.join(File.dirname(__FILE__), '../..', "tomte_protocol_spec_helper")

describe "A Consumer" do
  describe "regarding reliability" do

    #
    # Environment
    #

    include EventedSpec::AMQPSpec
    default_options AMQP_OPTS
    default_timeout 5

    amqp_before do
      @ep  = Tomte::Protocol::Endpoint.new(:bus => :test)

      @producer = @ep.producer(:exchange => "tomte.tests.integration.direct.exchange",
                               :reliable => true,
                               :exchange_options => {:auto_delete => true})

      @consumer = @ep.consumer(:queue_options => {:auto_delete => true})
    end


    def get_ready_and_listen(key, consume = true, &block)
      @ep.connect! do
        @producer.connect! do |p|
          @consumer.connect! do |c|
            c.bind("tomte.tests.integration.direct.exchange",
                    :routing_key => key)

            c.consume if consume
          end

          EventMachine.add_timer(0.5) do
            block.call p
          end
        end
      end
    end

    def close_and_done
      @producer.close { @consumer.close {  @ep.disconnect! { done }}}
    end


    describe "with a non mandatory, non immediate message" do
      it "should confirm when the message is delivered" do

        events   = Array.new
        @consumer.on_delivery do |payload|

          events << :received
        end

        get_ready_and_listen('good') do |p|

          errback = proc { events << :error }
          p.publish("", :persistent => false, :immediate => false,
                        :mandatory => false, :key => "good",
                        :errback => errback) do
            events << :confirmed
          end
        end

        delay(1) do
          expect(events).to include(:received)
          expect(events).to include(:confirmed)
          expect(events).not_to include(:error)
          close_and_done
        end
      end

      it "should confirm when the message is taken care of, even if is not deliverable" do
        events   = Array.new
        @consumer.on_delivery do |payload|

          events << :received
        end

        get_ready_and_listen('good') do |p|

          errback = proc { events << :error }
          p.publish("", :persistent => false, :immediate => false,
                        :mandatory => false, :key => "bad",
                        :errback => errback) do
            events << :confirmed
          end
        end

        delay(1) do
          expect(events).not_to include(:received)
          expect(events).to include(:confirmed)
          expect(events).not_to include(:error)
          close_and_done
        end
      end

    end

    describe "with a mandatory, non immediate message" do
      it "should confirm when the message is delivered to the queue" do
        events   = Array.new
        @consumer.on_delivery do |payload|

          events << :received
        end

        get_ready_and_listen('good') do |p|

          errback = proc { events << :error }
          p.publish("", :persistent => false, :immediate => false,
                        :mandatory => true, :key => "good",
                        :errback => errback) do
            events << :confirmed
          end
        end

        delay(1) do
          expect(events).to include(:received)
          expect(events).to include(:confirmed)
          expect(events).not_to include(:error)
          close_and_done
        end
      end

      it "should FAIL when the message not routable" do
        events   = Array.new
        @consumer.on_delivery do |payload|

          events << :received
        end

        get_ready_and_listen('good') do |p|

          errback = proc { events << :error }
          p.publish("", :persistent => false, :immediate => false,
                        :mandatory => true, :key => "bad",
                        :errback => errback) do
            events << :confirmed
          end
        end

        delay(1) do
          expect(events).not_to include(:received)
          expect(events).not_to include(:confirmed)
          expect(events).to include(:error)
          close_and_done
        end
      end

      it "should default to mandatory if nothing is given" do
        events   = Array.new
        @consumer.on_delivery do |payload|

          events << :received
        end

        get_ready_and_listen('good') do |p|

          errback = proc { events << :error }
          p.publish("", :persistent => false,  :key => "bad",
                        :errback => errback) do
            events << :confirmed
          end
        end

        delay(1) do
          expect(events).not_to include(:received)
          expect(events).not_to include(:confirmed)
          expect(events).to include(:error)
          close_and_done
        end
      end

    end


    describe "with a mandatory, immediate message" do
      it "should confirm when the message is delivered to the queue" do
        events   = Array.new
        @consumer.on_delivery do |payload|

          events << :received
        end

        get_ready_and_listen('good') do |p|

          errback = proc { events << :error }
          p.publish("", :persistent => false, :immediate => true,
                        :mandatory => true, :key => "good",
                        :errback => errback) do
            events << :confirmed
          end
        end

        delay(1) do
          expect(events).to include(:received)
          expect(events).to include(:confirmed)
          expect(events).not_to include(:error)
          close_and_done
        end
      end

      it "should FAIL when the message not routable" do
        events   = Array.new
        @consumer.on_delivery do |payload|

          events << :received
        end

        get_ready_and_listen('good', false) do |p|

          errback = proc { events << :error }
          p.publish("", :persistent => false, :immediate => true,
                        :mandatory => true, :key => "good",
                        :errback => errback) do
            events << :confirmed
          end
        end

        delay(1) do
          expect(events).not_to include(:received)
          expect(events).not_to include(:confirmed)
          expect(events).to include(:error)
          close_and_done
        end
      end
    end


    it "should ack multiple messages" do
      events   = Array.new
      @consumer.on_delivery do |payload|
        events << :"received"
      end

      get_ready_and_listen('good') do |p|

        errback = proc { events << :error }
        10.times do |n|
          p.publish("", :persistent => false, :immediate => true,
                        :mandatory => true, :key => "good",
                        :errback => errback) do
            events << :"confirmed-#{n}"
          end
        end
      end

      delay(1) do
        10.times do |n|
          expect(events).to include(:"confirmed-#{n}")
        end

        expect(events).to include(:received)
        expect(events).not_to include(:error)
        close_and_done
      end
    end

  end
end