require File.join(File.dirname(__FILE__), '../..', "tomte_protocol_spec_helper")
require 'tomte/protocol/enveloper'
require 'tomte/message'

class MyDoc
  include Tomte::Document

  key :foo, String
  key :bar, String, :default => "barz"

end


class GenericSerializable
  attr_accessor :foo
  attr_accessor :bar

  def initialize(attrs)
    @foo, @bar = [attrs[:foo], attrs[:bar]]
  end

  def self.from_tomte(attrs)
    new(attrs)
  end

  def to_tomte
    {:foo => @foo, :bar => @bar}
  end
end



describe "the Generic Enveloper " do

  before :each do
    @e = Tomte::Protocol::Enveloper::Generic.new(:serialization => :bson)

    @doc = MyDoc.new(:foo => "fooz")

    @msg = Tomte::Message.new(
        :metadata => Tomte::Message::Metadata.new(:message_id => "1234"),
        :payload => @doc,
        :headers => {:a => 42, :b => 23}
    )

    @generic = GenericSerializable.new(:foo => 42, :bar => "poo")
  end

  it 'should envelope, starting from a Message' do
    res = @e.local_to_wire(@msg)
    expect(res).to be_a_kind_of Tomte::Protocol::Enveloper::WireResponse
    expect(res.body.to_s).to be_a_kind_of String
    expect(res.metadata).to be_a_kind_of Hash
    expect(res.metadata[:message_id]).to eq '1234'
    expect(res.metadata[:headers]).to be_a_kind_of Hash
    expect(res.metadata[:headers][:a]).to eq 42
  end

  it 'should envelope, starting from a Document' do
    res = @e.local_to_wire(@doc, :message_id => "1234", :headers => {:a => 42, :b => 23})
    expect(res).to be_a_kind_of Tomte::Protocol::Enveloper::WireResponse
    expect(res.body.to_s).to be_a_kind_of String
    expect(res.metadata).to be_a_kind_of Hash
    expect(res.metadata[:message_id]).to eq '1234'
    expect(res.metadata[:headers]).to be_a_kind_of Hash
    expect(res.metadata[:headers][:a]).to eq 42
  end

  it 'should envelope, starting from any Hash' do
    res = @e.local_to_wire({:a => 1, :b => 2}, :message_id => "1234", :headers => {:a => 42, :b => 23})
    expect(res).to be_a_kind_of Tomte::Protocol::Enveloper::WireResponse
    expect(res.body.to_s).to be_a_kind_of String
    expect(res.metadata).to be_a_kind_of Hash
    expect(res.metadata[:message_id]).to eq '1234'
    expect(res.metadata[:headers]).to be_a_kind_of Hash
    expect(res.metadata[:headers][:a]).to eq 42
  end

  it 'should envelope, starting from any document that can be made an  Hash' do
    #tbd
  end

  it 'should NOT envelope, starting from any generic object' do
    expect { @e.local_to_wire("Hello World")}.to raise_error(Tomte::Protocol::Enveloper::Error)
  end

  it 'should envelope/deenvelope, starting from a Message' do
    res = @e.local_to_wire(@msg)
    orig = @e.wire_to_local(res.body, res.metadata)
    payload = orig.payload
    metadata = orig.metadata

    expect(payload).to be_a_kind_of Tomte::Document
    expect(payload.foo).to eq "fooz"
    expect(payload.bar).to eq "barz"

    expect(metadata.headers[:a]).to eq 42
    expect(metadata.message_id).to eq "1234"

  end

  it 'should envelope/deenvelope, starting from a Document' do
    res = @e.local_to_wire(@doc, :message_id => "1234", :headers => {:a => 42, :b => 23})
    orig = @e.wire_to_local(res.body, res.metadata)
    payload = orig.payload
    metadata = orig.metadata

    expect(payload).to be_a_kind_of Tomte::Document
    expect(payload.foo).to eq "fooz"
    expect(payload.bar).to eq "barz"

    expect(metadata.headers[:a]).to eq 42
    expect(metadata.message_id).to eq "1234"
  end

  it 'should envelope/deenvelope, starting from a hash' do
    res = @e.local_to_wire({:a => 1, :b => 2}, :message_id => "1234", :headers => {:a => 42, :b => 23})

    orig = @e.wire_to_local(res.body, res.metadata)
    payload = orig.payload
    metadata = orig.metadata

    expect(payload).to be_a_kind_of Hash
    expect(payload[:a]).to eq 1
    expect(payload[:b]).to eq 2

    expect(metadata.headers[:a]).to eq 42
    expect(metadata.message_id).to eq "1234"
  end

  it 'should envelope/deenvelope, starting from a generic serializable' do
    res = @e.local_to_wire(@generic)

    orig = @e.wire_to_local(res.body, res.metadata)

    payload = orig.payload
    metadata = orig.metadata

    expect(payload).to be_a_kind_of @generic.class
    expect(payload.foo).to eq 42
    expect(payload.bar).to eq 'poo'
  end


end